==> sli-plots-0.0.5.tgz: package/addon/components/bar-hover.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import { bisector } from "d3-array";
import { throttle } from '@ember/runloop';

export default class MultiScrubberComponent extends Component {
  get bisectDate() {
    return bisector(d => d.at).left;
  }

  @tracked fadeState = "opacity-0";
  @tracked hoverTextValues;
  @tracked textSizeElement;
  @tracked textBox;

  // ------
  // Params
  // ------
  // Required: chart, width, datasets, range, xScale, yScale, colors

  // Any units text for the values
  get units() {
    return this.args.units ? this.args.units : '';
  }
  
  get valueKey() {
    return this.args.valueKey ? this.args.valueKey : 'value'
  }

  get timeKey() {
    return this.args.timeKey ? this.args.timeKey : 'at'
  }

  get primaryDataset() {
    return this.args.datasets ? this.args.datasets[0] : []
  }

  get textLineHeight() {
    return this.args.textLineHeight ? this.args.textLineHeight : 15
  }

  get chartWidth() { // chart width
    return this.args.width ? this.args.width : 200;
  }

  get textBoxStrokeWidth() {
    return this.args.textBoxStrokeWidth ? this.args.textBoxStrokeWidth : '1px';
  }

  get textBoxStroke() {
    return this.args.textBoxStroke ? this.args.textBoxStroke : 'black';
  }

  get minAggregate() {
    return this.args.minAggregate ? this.args.minAggregate : 60 * 60 * 1000;
  }

  get aggregatedTime() {
    return this.args.aggregatedTime && this.args.aggregatedTime > this.minAggregate ? this.args.aggregatedTime : this.minAggregate;
  }

  get hideTime() {
    return this.args.hideTime ? this.args.hideTime : false;
  }

  get textBoxPadding() {
    return this.args.textBoxPadding ? this.args.textBoxPadding : 20;
  }

  // --------
  // Computed
  // --------

  get isMultiScale() {
    return typeof this.args.xScale === 'object'
  }

  get barWidth() {
    const xScale = this.isMultiScale ? this.args.xScale[0] : this.args.xScale;
    return xScale(new Date(this.args.range[0].getTime() + this.aggregatedTime)) - xScale(this.args.range[0]);
  };

  // -----
  // functions
  // -----
  updateHoverInfo = (e) => {
    const mousePos = this.mouseOrTouchPosition(e);    
    if( !isNaN(mousePos) ) {
      const dataPoints = this.findNearestDatapoints(mousePos).filter(dp => dp);
      const yLoc = this.findMin(dataPoints, 'y');
      this.textBox = this.findTextXLocBoundedBySvg(dataPoints, yLoc);

      this.hoverTextValues = dataPoints.map((datapoint, index) => {
        return {
          ...datapoint,
          y: this.textBox.y + this.textBoxPadding + (index * this.textLineHeight + 10),
          x: this.textBox.x + this.textBoxPadding
        }
      })
    }
  }

  findTextXLocBoundedBySvg(dataPoints, yLoc) {
    if(!dataPoints.length) return 0
    const pointWithLargestText = dataPoints.reduce((max, dataPoint) => !max || dataPoint.text.length > max ? dataPoint : max);
    const textSize = this.calculateTextDimensions(pointWithLargestText.text);
    const textBoxWidth = this.textBoxPadding * 2 + textSize.width
    const barCenterPoint = dataPoints[0].x + (this.barWidth/2);
    const leftSide = barCenterPoint - (textBoxWidth)/2 ;
    const rightSide = dataPoints[0].x + textSize.width + this.textBoxPadding;
    const y = yLoc - (this.textBoxPadding * 2 + (this.textLineHeight * dataPoints.length))

    return {
      x: leftSide < 5 ? 5 : rightSide > (this.chartWidth - 5) ? this.chartWidth - textBoxWidth - 5 : leftSide,
      y: y < 5 ? 5 : y,
      width: textBoxWidth,
      height: this.textLineHeight * dataPoints.length + this.textBoxPadding * 2
    }
  }

  calculateTextDimensions(text) {
    this.textSizeElement.innerHTML = text;
    return {
      width: this.textSizeElement.clientWidth + 1,
      height: this.textSizeElement.clientHeight + 1
    }
  }

  initElement = (element) => {
    this.textSizeElement = element;
  }

  findMin(array, field) {
    return array.reduce((max, obj) => {
      return obj && obj[field] < max ? obj[field] : max
    }, this.args.yScale(0))
  }

  findNearestDatapoints(mousePos) {
    if (!this.args.datasets) return [{x: "", y: ""}];

    return this.args.datasets.map((dataset, index) => {
      const xScale = this.isMultiScale ? this.args.xScale[index] : this.args.xScale
      const x0 = xScale.invert(mousePos); // x position within the element.
      if (dataset.length === 0 || x0 < dataset[0].at) return // out of bounds
      const i = this.bisectDate(dataset, x0, 1);
      const color = this.args.colors ? this.args.colors[index] : 'black';
      return this.getDatapointInfo(x0, xScale, dataset[i - 1], color);
    });
  }

  getDatapointInfo(x0, xScale, datapoint, color) {
    const isInRange = x0 < new Date(datapoint.at.getTime() + this.aggregatedTime)
    if(!datapoint || !isInRange) return
    const rawValue = datapoint ? datapoint[this.valueKey] : "";
    const value = rawValue ? `${rawValue.toFixed(2)}${this.units}` : "";
    const unformattedAt = datapoint ? datapoint[this.timeKey] : "";
    const at = unformattedAt ? this.formatDate(unformattedAt) : "";
    const x = xScale(unformattedAt) + (this.textAnchor === 'end' ? 25 : 0);
    return {
      ...datapoint,
      x: x < 0 ? 0 : x,
      y: this.args.yScale(rawValue) - this.textLineHeight,
      at,
      value,
      color,
      text: `${value}, ${at}`
    };
  }

  mouseOrTouchPosition(event) {
    const clientX = event.type.includes("touch") ? event.targetTouches[0].clientX : event.clientX;
    return clientX - this.args.chart.getBoundingClientRect().left;
  }

  formatDate(date) {
    const dateObj = new Date(date);
    const hours = dateObj.getHours();
    const minutes = dateObj.getMinutes();
    const time = `, ${hours < 12 ? hours : hours - 12}:${('0' + minutes).slice(-2)} ${hours < 12 ? "am" : "pm"}`
    return `${dateObj.getMonth()+1}-${dateObj.getDate()}-${dateObj.getFullYear()}${this.hideTime ? "" : time}`;
  }

  mousemove = (e) => {
    throttle(this, this.updateHoverInfo, e, 20);
  }

  mouseout = () => {
    this.fadeState = "opacity-0";
  }

  mouseover = (e) => {
    this.fadeState = "opacity-100";
  }
}

==> sli-plots-0.0.7.tgz: package/addon/components/bar-hover.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import { bisector } from "d3-array";
import { throttle } from '@ember/runloop';

export default class MultiScrubberComponent extends Component {
  get bisectDate() {
    return bisector(d => d.at).left;
  }

  @tracked fadeState = "opacity-0";
  @tracked hoverTextValues;
  @tracked textSizeElement;
  @tracked textBox;

  // ------
  // Params
  // ------
  // Required: chart, width, datasets, range, xScale, yScale, colors

  // Any units text for the values
  get units() {
    return this.args.units ? this.args.units : '';
  }
  
  get valueKey() {
    return this.args.valueKey ? this.args.valueKey : 'value'
  }

  get timeKey() {
    return this.args.timeKey ? this.args.timeKey : 'at'
  }

  get primaryDataset() {
    return this.args.datasets ? this.args.datasets[0] : []
  }

  get textLineHeight() {
    return this.args.textLineHeight ? this.args.textLineHeight : 15
  }

  get chartWidth() { // chart width
    return this.args.width ? this.args.width : 200;
  }

  get textBoxStrokeWidth() {
    return this.args.textBoxStrokeWidth ? this.args.textBoxStrokeWidth : '1px';
  }

  get textBoxStroke() {
    return this.args.textBoxStroke ? this.args.textBoxStroke : 'black';
  }

  get minAggregate() {
    return this.args.minAggregate ? this.args.minAggregate : 60 * 60 * 1000;
  }

  get aggregatedTime() {
    return this.args.aggregatedTime && this.args.aggregatedTime > this.minAggregate ? this.args.aggregatedTime : this.minAggregate;
  }

  get hideTime() {
    return this.args.hideTime ? this.args.hideTime : false;
  }

  get textBoxPadding() {
    return this.args.textBoxPadding ? this.args.textBoxPadding : 20;
  }

  get displayDecimalPlace() {
    return this.args.displayDecimalPlace || 3;
  }

  // --------
  // Computed
  // --------

  get isMultiScale() {
    return typeof this.args.xScale === 'object'
  }

  get barWidth() {
    const xScale = this.isMultiScale ? this.args.xScale[0] : this.args.xScale;
    return xScale(new Date(this.args.range[0].getTime() + this.aggregatedTime)) - xScale(this.args.range[0]);
  };

  // -----
  // functions
  // -----
  updateHoverInfo = (e) => {
    const mousePos = this.mouseOrTouchPosition(e);    
    if( !isNaN(mousePos) ) {
      const dataPoints = this.findNearestDatapoints(mousePos).filter(dp => dp);
      const yLoc = this.findMin(dataPoints, 'y');
      this.textBox = this.findTextXLocBoundedBySvg(dataPoints, yLoc);

      this.hoverTextValues = dataPoints.map((datapoint, index) => {
        return {
          ...datapoint,
          y: this.textBox.y + this.textBoxPadding + (index * this.textLineHeight + 10),
          x: this.textBox.x + this.textBoxPadding
        }
      })
    }
  }

  findTextXLocBoundedBySvg(dataPoints, yLoc) {
    if(!dataPoints.length) return 0
    const pointWithLargestText = dataPoints.reduce((max, dataPoint) => !max || dataPoint.text.length > max ? dataPoint : max);
    const textSize = this.calculateTextDimensions(pointWithLargestText.text);
    const textBoxWidth = this.textBoxPadding * 2 + textSize.width
    const barCenterPoint = dataPoints[0].x + (this.barWidth/2);
    const leftSide = barCenterPoint - (textBoxWidth)/2 ;
    const rightSide = dataPoints[0].x + textSize.width + this.textBoxPadding;
    const y = yLoc - (this.textBoxPadding * 2 + (this.textLineHeight * dataPoints.length))
    return {
      x: leftSide < 5 ? 5 : rightSide > (this.chartWidth - 5) ? this.chartWidth - textBoxWidth - 5 : leftSide,
      y: y < 5 ? 5 : isNaN(y) ? -250 : y,
      width: textBoxWidth,
      height: this.textLineHeight * dataPoints.length + this.textBoxPadding * 2
    }
  }

  calculateTextDimensions(text) {
    this.textSizeElement.innerHTML = text;
    return {
      width: this.textSizeElement.clientWidth + 1,
      height: this.textSizeElement.clientHeight + 1
    }
  }

  initElement = (element) => {
    this.textSizeElement = element;
  }

  findMin(array, field) {
    return array.reduce((max, obj) => {
      return obj && obj[field] < max ? obj[field] : max
    }, this.args.yScale(0))
  }

  findNearestDatapoints(mousePos) {
    if (!this.args.datasets) return [{x: "", y: ""}];

    return this.args.datasets.map((dataset, index) => {
      const xScale = this.isMultiScale ? this.args.xScale[index] : this.args.xScale
      const x0 = xScale.invert(mousePos); // x position within the element.
      if (dataset.length === 0 || x0 < dataset[0].at) return // out of bounds
      const i = this.bisectDate(dataset, x0, 1);
      const color = this.args.colors ? this.args.colors[index] : 'black';
      return this.getDatapointInfo(x0, xScale, dataset[i - 1], color);
    });
  }

  getDatapointInfo(x0, xScale, datapoint, color) {
    const isInRange = x0 < new Date(datapoint.at.getTime() + this.aggregatedTime)
    if(!datapoint || !isInRange) return
    const rawValue = datapoint ? datapoint[this.valueKey] : "";
    const value = rawValue ? `${rawValue.toFixed(this.displayDecimalPlace)}${this.units}` : "";
    const unformattedAt = datapoint ? datapoint[this.timeKey] : "";
    const at = unformattedAt ? this.formatDate(unformattedAt) : "";
    const x = xScale(unformattedAt) + (this.textAnchor === 'end' ? 25 : 0);
    return {
      ...datapoint,
      x: x < 0 ? 0 : x,
      y: this.args.yScale(rawValue) - this.textLineHeight,
      at,
      value,
      color,
      text: `${value}, ${at}`
    };
  }

  mouseOrTouchPosition(event) {
    const clientX = event.type.includes("touch") ? event.targetTouches[0].clientX : event.clientX;
    return clientX - this.args.chart.getBoundingClientRect().left;
  }

  formatDate(date) {
    const dateObj = new Date(date);
    const hours = dateObj.getHours();
    const minutes = dateObj.getMinutes();
    const time = `, ${hours < 12 ? hours : hours - 12}:${('0' + minutes).slice(-2)} ${hours < 12 ? "am" : "pm"}`
    return `${dateObj.getMonth()+1}-${dateObj.getDate()}-${dateObj.getFullYear()}${this.hideTime ? "" : time}`;
  }

  mousemove = (e) => {
    throttle(this, this.updateHoverInfo, e, 20);
  }

  mouseout = () => {
    this.fadeState = "opacity-0";
  }

  mouseover = (e) => {
    this.fadeState = "opacity-100";
  }
}
